"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseLocFile = void 0;
const node_core_library_1 = require("@rushstack/node-core-library");
const ResxReader_1 = require("./ResxReader");
const LOC_JSON_SCHEMA = node_core_library_1.JsonSchema.fromFile(`${__dirname}/schemas/locJson.schema.json`);
const parseCache = new Map();
/**
 * @public
 */
function parseLocFile(options) {
    const fileCacheKey = `${options.filePath}?${options.resxNewlineNormalization || 'none'}`;
    if (parseCache.has(fileCacheKey)) {
        const entry = parseCache.get(fileCacheKey);
        if (entry.content === options.content) {
            return entry.parsedFile;
        }
    }
    let parsedFile;
    if (/\.resx$/i.test(options.filePath)) {
        parsedFile = (0, ResxReader_1.readResxAsLocFile)(options.content, {
            terminal: options.terminal,
            resxFilePath: options.filePath,
            newlineNormalization: options.resxNewlineNormalization,
            warnOnMissingComment: !options.ignoreMissingResxComments
        });
    }
    else if (/\.(resx|loc)\.json$/i.test(options.filePath)) {
        parsedFile = node_core_library_1.JsonFile.parseString(options.content);
        try {
            LOC_JSON_SCHEMA.validateObject(parsedFile, options.filePath);
        }
        catch (e) {
            options.terminal.writeError(`The loc file is invalid. Error: ${e}`);
        }
    }
    else if (/\.resjson$/i.test(options.filePath)) {
        const resjsonFile = node_core_library_1.JsonFile.parseString(options.content);
        parsedFile = {};
        const comments = new Map();
        for (const [key, value] of Object.entries(resjsonFile)) {
            if (key.startsWith('_') && key.endsWith('.comment')) {
                const commentKey = key.substring(1, key.length - '.comment'.length);
                comments.set(commentKey, value);
            }
            else {
                parsedFile[key] = { value };
            }
        }
        const orphanComments = [];
        for (const [key, comment] of comments) {
            if (parsedFile[key]) {
                parsedFile[key].comment = comment;
            }
            else {
                orphanComments.push(key);
            }
        }
        if (orphanComments.length > 0) {
            options.terminal.writeErrorLine('The resjson file is invalid. Comments exist for the following string keys ' +
                `that don't have values: ${orphanComments.join(', ')}.`);
        }
    }
    else {
        throw new Error(`Unsupported file extension in file: ${options.filePath}`);
    }
    parseCache.set(fileCacheKey, { content: options.content, parsedFile });
    return parsedFile;
}
exports.parseLocFile = parseLocFile;
//# sourceMappingURL=LocFileParser.js.map